@@ -47,37 +47,53 @@ typedef enum {
 	SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY
 } SoupCookieJarAcceptPolicy;
 
+SOUP_AVAILABLE_IN_2_24
 GType                     soup_cookie_jar_get_type                    (void);
+SOUP_AVAILABLE_IN_2_24
 SoupCookieJar *           soup_cookie_jar_new                         (void);
-#ifndef LIBSOUP_DISABLE_DEPRECATED
-void                      soup_cookie_jar_save                        (SoupCookieJar             *jar);
-#endif
+SOUP_AVAILABLE_IN_2_24
 char          *           soup_cookie_jar_get_cookies                 (SoupCookieJar             *jar,
 								       SoupURI                   *uri,
 								       gboolean                   for_http);
+SOUP_AVAILABLE_IN_2_40
 GSList        *           soup_cookie_jar_get_cookie_list             (SoupCookieJar             *jar,
 								       SoupURI                   *uri,
 								       gboolean                   for_http);
+SOUP_AVAILABLE_IN_2_24
 void                      soup_cookie_jar_set_cookie                  (SoupCookieJar             *jar,
 								       SoupURI                   *uri,
 								       const char                *cookie);
+SOUP_AVAILABLE_IN_2_30
 void                      soup_cookie_jar_set_cookie_with_first_party (SoupCookieJar             *jar,
 								       SoupURI                   *uri,
 								       SoupURI                   *first_party,
 								       const char                *cookie);
+SOUP_AVAILABLE_IN_2_26
 void                      soup_cookie_jar_add_cookie                  (SoupCookieJar             *jar,
 								       SoupCookie                *cookie);
+SOUP_AVAILABLE_IN_2_40
 void                      soup_cookie_jar_add_cookie_with_first_party (SoupCookieJar             *jar,
 								       SoupURI                   *first_party,
 								       SoupCookie                *cookie);
+SOUP_AVAILABLE_IN_2_26
 void                      soup_cookie_jar_delete_cookie               (SoupCookieJar             *jar,
 								       SoupCookie                *cookie);
+SOUP_AVAILABLE_IN_2_26
 GSList        *           soup_cookie_jar_all_cookies                 (SoupCookieJar             *jar);
+SOUP_AVAILABLE_IN_2_30
 void                      soup_cookie_jar_set_accept_policy           (SoupCookieJar             *jar,
 								       SoupCookieJarAcceptPolicy  policy);
+SOUP_AVAILABLE_IN_2_30
 SoupCookieJarAcceptPolicy soup_cookie_jar_get_accept_policy           (SoupCookieJar             *jar);
+SOUP_AVAILABLE_IN_2_40
 gboolean                  soup_cookie_jar_is_persistent               (SoupCookieJar             *jar);
 
+#ifndef SOUP_DISABLE_DEPRECATED
+SOUP_AVAILABLE_IN_2_24
+SOUP_DEPRECATED_IN_2_26
+void                      soup_cookie_jar_save                        (SoupCookieJar             *jar);
+#endif
+
 G_END_DECLS
 
 #endif /* SOUP_COOKIE_JAR_H */
