@@ -73,7 +73,7 @@ server_callback (SoupServer *server, SoupMessage *msg,
 	if (!strcmp (path, "/slow")) {
 		soup_server_pause_message (server, msg);
 		g_object_set_data (G_OBJECT (msg), "server", server);
-		soup_add_timeout (soup_server_get_async_context (server),
+		soup_add_timeout (g_main_context_get_thread_default (),
 				  1000, timeout_finish_message, msg);
 	}
 
@@ -91,7 +91,6 @@ server_callback (SoupServer *server, SoupMessage *msg,
 
 /* Host header handling: client must be able to override the default
  * value, server must be able to recognize different Host values.
- * #539803.
  */
 static void
 do_host_test (void)
@@ -99,7 +98,7 @@ do_host_test (void)
 	SoupSession *session;
 	SoupMessage *one, *two;
 
-	debug_printf (1, "Host handling\n");
+	g_test_bug ("539803");
 
 	session = soup_test_session_new (SOUP_TYPE_SESSION_SYNC, NULL);
 
@@ -112,42 +111,24 @@ do_host_test (void)
 
 	soup_test_session_abort_unref (session);
 
-	if (!SOUP_STATUS_IS_SUCCESSFUL (one->status_code)) {
-		debug_printf (1, "  Message 1 failed: %d %s\n",
-			      one->status_code, one->reason_phrase);
-		errors++;
-	} else if (strcmp (one->response_body->data, "index") != 0) {
-		debug_printf (1, "  Unexpected response to message 1: '%s'\n",
-			      one->response_body->data);
-		errors++;
-	}
+	soup_test_assert_message_status (one, SOUP_STATUS_OK);
+	g_assert_cmpstr (one->response_body->data, ==, "index");
 	g_object_unref (one);
 
-	if (!SOUP_STATUS_IS_SUCCESSFUL (two->status_code)) {
-		debug_printf (1, "  Message 2 failed: %d %s\n",
-			      two->status_code, two->reason_phrase);
-		errors++;
-	} else if (strcmp (two->response_body->data, "foo-index") != 0) {
-		debug_printf (1, "  Unexpected response to message 2: '%s'\n",
-			      two->response_body->data);
-		errors++;
-	}
+	soup_test_assert_message_status (two, SOUP_STATUS_OK);
+	g_assert_cmpstr (two->response_body->data, ==, "foo-index");
 	g_object_unref (two);
 }
 
 /* Dropping the application's ref on the session from a callback
  * should not cause the session to be freed at an incorrect time.
- * (This test will crash if it fails.) #533473
+ * (This test will crash if it fails.)
  */
 static void
 cu_one_completed (SoupSession *session, SoupMessage *msg, gpointer loop)
 {
 	debug_printf (2, "  Message 1 completed\n");
-	if (msg->status_code != SOUP_STATUS_CANT_CONNECT) {
-		debug_printf (1, "  Unexpected status on Message 1: %d %s\n",
-			      msg->status_code, msg->reason_phrase);
-		errors++;
-	}
+	soup_test_assert_message_status (msg, SOUP_STATUS_CANT_CONNECT);
 	g_object_unref (session);
 }
 
@@ -162,11 +143,7 @@ static void
 cu_two_completed (SoupSession *session, SoupMessage *msg, gpointer loop)
 {
 	debug_printf (2, "  Message 2 completed\n");
-	if (msg->status_code != SOUP_STATUS_CANT_CONNECT) {
-		debug_printf (1, "  Unexpected status on Message 2: %d %s\n",
-			      msg->status_code, msg->reason_phrase);
-		errors++;
-	}
+	soup_test_assert_message_status (msg, SOUP_STATUS_CANT_CONNECT);
 	g_idle_add (cu_idle_quit, loop); 
 }
 
@@ -174,35 +151,28 @@ static void
 do_callback_unref_test (void)
 {
 	SoupServer *bad_server;
-	SoupAddress *addr;
 	SoupSession *session;
 	SoupMessage *one, *two;
 	GMainLoop *loop;
-	char *bad_uri;
+	SoupURI *bad_uri;
 
-	debug_printf (1, "\nCallback unref handling (msg api)\n");
+	g_test_bug ("533473");
 
 	/* Get a guaranteed-bad URI */
-	addr = soup_address_new ("127.0.0.1", SOUP_ADDRESS_ANY_PORT);
-	soup_address_resolve_sync (addr, NULL);
-	bad_server = soup_server_new (SOUP_SERVER_INTERFACE, addr,
-				      NULL);
-	g_object_unref (addr);
-
-	bad_uri = g_strdup_printf ("http://127.0.0.1:%u/",
-				   soup_server_get_port (bad_server));
-	g_object_unref (bad_server);
+	bad_server = soup_test_server_new (SOUP_TEST_SERVER_DEFAULT);
+	bad_uri = soup_test_server_get_uri (bad_server, "http", NULL);
+	soup_test_server_quit_unref (bad_server);
 
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
 	g_object_add_weak_pointer (G_OBJECT (session), (gpointer *)&session);
 
 	loop = g_main_loop_new (NULL, TRUE);
 
-	one = soup_message_new ("GET", bad_uri);
+	one = soup_message_new_from_uri ("GET", bad_uri);
 	g_object_add_weak_pointer (G_OBJECT (one), (gpointer *)&one);
-	two = soup_message_new ("GET", bad_uri);
+	two = soup_message_new_from_uri ("GET", bad_uri);
 	g_object_add_weak_pointer (G_OBJECT (two), (gpointer *)&two);
-	g_free (bad_uri);
+	soup_uri_free (bad_uri);
 
 	soup_session_queue_message (session, one, cu_one_completed, loop);
 	soup_session_queue_message (session, two, cu_two_completed, loop);
@@ -210,22 +180,19 @@ do_callback_unref_test (void)
 	g_main_loop_run (loop);
 	g_main_loop_unref (loop);
 
+	g_assert_null (session);
 	if (session) {
 		g_object_remove_weak_pointer (G_OBJECT (session), (gpointer *)&session);
-		debug_printf (1, "  Session not destroyed?\n");
-		errors++;
 		g_object_unref (session);
 	}
+	g_assert_null (one);
 	if (one) {
 		g_object_remove_weak_pointer (G_OBJECT (one), (gpointer *)&one);
-		debug_printf (1, "  Message 1 not destroyed?\n");
-		errors++;
 		g_object_unref (one);
 	}
+	g_assert_null (two);
 	if (two) {
 		g_object_remove_weak_pointer (G_OBJECT (two), (gpointer *)&two);
-		debug_printf (1, "  Message 2 not destroyed?\n");
-		errors++;
 		g_object_unref (two);
 	}
 
@@ -239,14 +206,8 @@ cur_one_completed (GObject *source, GAsyncResult *result, gpointer session)
 	GError *error = NULL;
 
 	debug_printf (2, "  Request 1 completed\n");
-	if (soup_request_send_finish (one, result, &error)) {
-		debug_printf (1, "  Request 1 succeeded?\n");
-		errors++;
-	} else if (!g_error_matches (error, SOUP_HTTP_ERROR, SOUP_STATUS_CANT_CONNECT)) {
-		debug_printf (1, "  Unexpected error on Request 1: %s\n",
-			      error->message);
-		errors++;
-	}
+	soup_request_send_finish (one, result, &error);
+	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CONNECTION_REFUSED);
 	g_clear_error (&error);
 
 	g_object_unref (session);
@@ -266,14 +227,8 @@ cur_two_completed (GObject *source, GAsyncResult *result, gpointer loop)
 	GError *error = NULL;
 
 	debug_printf (2, "  Request 2 completed\n");
-	if (soup_request_send_finish (two, result, &error)) {
-		debug_printf (1, "  Request 2 succeeded?\n");
-		errors++;
-	} else if (!g_error_matches (error, SOUP_HTTP_ERROR, SOUP_STATUS_CANT_CONNECT)) {
-		debug_printf (1, "  Unexpected error on Request 2: %s\n",
-			      error->message);
-		errors++;
-	}
+	soup_request_send_finish (two, result, &error);
+	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CONNECTION_REFUSED);
 	g_clear_error (&error);
 
 	g_idle_add (cur_idle_quit, loop); 
@@ -283,24 +238,15 @@ static void
 do_callback_unref_req_test (void)
 {
 	SoupServer *bad_server;
-	SoupAddress *addr;
 	SoupSession *session;
 	SoupRequest *one, *two;
 	GMainLoop *loop;
-	char *bad_uri;
-
-	debug_printf (1, "\nCallback unref handling (request api)\n");
+	SoupURI *bad_uri;
 
 	/* Get a guaranteed-bad URI */
-	addr = soup_address_new ("127.0.0.1", SOUP_ADDRESS_ANY_PORT);
-	soup_address_resolve_sync (addr, NULL);
-	bad_server = soup_server_new (SOUP_SERVER_INTERFACE, addr,
-				      NULL);
-	g_object_unref (addr);
-
-	bad_uri = g_strdup_printf ("http://127.0.0.1:%u/",
-				   soup_server_get_port (bad_server));
-	g_object_unref (bad_server);
+	bad_server = soup_test_server_new (SOUP_TEST_SERVER_DEFAULT);
+	bad_uri = soup_test_server_get_uri (bad_server, "http", NULL);
+	soup_test_server_quit_unref (bad_server);
 
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC,
 					 SOUP_SESSION_USE_THREAD_CONTEXT, TRUE,
@@ -309,11 +255,11 @@ do_callback_unref_req_test (void)
 
 	loop = g_main_loop_new (NULL, TRUE);
 
-	one = soup_session_request (session, bad_uri, NULL);
+	one = soup_session_request_uri (session, bad_uri, NULL);
 	g_object_add_weak_pointer (G_OBJECT (one), (gpointer *)&one);
-	two = soup_session_request (session, bad_uri, NULL);
+	two = soup_session_request_uri (session, bad_uri, NULL);
 	g_object_add_weak_pointer (G_OBJECT (two), (gpointer *)&two);
-	g_free (bad_uri);
+	soup_uri_free (bad_uri);
 
 	soup_request_send_async (one, NULL, cur_one_completed, session);
 	g_object_unref (one);
@@ -323,22 +269,19 @@ do_callback_unref_req_test (void)
 	g_main_loop_run (loop);
 	g_main_loop_unref (loop);
 
+	g_assert_null (session);
 	if (session) {
 		g_object_remove_weak_pointer (G_OBJECT (session), (gpointer *)&session);
-		debug_printf (1, "  Session not destroyed?\n");
-		errors++;
 		g_object_unref (session);
 	}
+	g_assert_null (one);
 	if (one) {
 		g_object_remove_weak_pointer (G_OBJECT (one), (gpointer *)&one);
-		debug_printf (1, "  Request 1 not destroyed?\n");
-		errors++;
 		g_object_unref (one);
 	}
+	g_assert_null (two);
 	if (two) {
 		g_object_remove_weak_pointer (G_OBJECT (two), (gpointer *)&two);
-		debug_printf (1, "  Request 2 not destroyed?\n");
-		errors++;
 		g_object_unref (two);
 	}
 
@@ -347,20 +290,19 @@ do_callback_unref_req_test (void)
 
 /* SoupSession should clean up all signal handlers on a message after
  * it is finished, allowing the message to be reused if desired.
- * #559054
  */
 static void
 ensure_no_signal_handlers (SoupMessage *msg, guint *signal_ids, guint n_signal_ids)
 {
 	int i;
+	guint id;
 
 	for (i = 0; i < n_signal_ids; i++) {
-		if (g_signal_handler_find (msg, G_SIGNAL_MATCH_ID, signal_ids[i],
-					   0, NULL, NULL, NULL)) {
-			debug_printf (1, "    Message has handler for '%s'\n",
-				      g_signal_name (signal_ids[i]));
-			errors++;
-		}
+		id = g_signal_handler_find (msg, G_SIGNAL_MATCH_ID, signal_ids[i],
+					    0, NULL, NULL, NULL);
+		soup_test_assert (id == 0,
+				  "message has handler for '%s'",
+				  g_signal_name (signal_ids[i]));
 	}
 }
 
@@ -383,7 +325,7 @@ do_msg_reuse_test (void)
 	SoupURI *uri;
 	guint *signal_ids, n_signal_ids;
 
-	debug_printf (1, "\nSoupMessage reuse\n");
+	g_test_bug ("559054");
 
 	signal_ids = g_signal_list_ids (SOUP_TYPE_MESSAGE, &n_signal_ids);
 
@@ -401,10 +343,7 @@ do_msg_reuse_test (void)
 	soup_message_set_uri (msg, uri);
 	soup_uri_free (uri);
 	soup_session_send_message (session, msg);
-	if (!soup_uri_equal (soup_message_get_uri (msg), base_uri)) {
-		debug_printf (1, "    Message did not get redirected!\n");
-		errors++;
-	}
+	g_assert_true (soup_uri_equal (soup_message_get_uri (msg), base_uri));
 	ensure_no_signal_handlers (msg, signal_ids, n_signal_ids);
 
 	debug_printf (1, "  Auth message\n");
@@ -412,10 +351,7 @@ do_msg_reuse_test (void)
 	soup_message_set_uri (msg, uri);
 	soup_uri_free (uri);
 	soup_session_send_message (session, msg);
-	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
-		debug_printf (1, "    Message did not get authenticated!\n");
-		errors++;
-	}
+	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
 	ensure_no_signal_handlers (msg, signal_ids, n_signal_ids);
 
 	/* One last try to make sure the auth stuff got cleaned up */
@@ -429,16 +365,12 @@ do_msg_reuse_test (void)
 	g_free (signal_ids);
 }
 
-/* Handle unexpectedly-early aborts. #596074, #618641 */
+/* Handle unexpectedly-early aborts. */
 static void
 ea_msg_completed_one (SoupSession *session, SoupMessage *msg, gpointer loop)
 {
 	debug_printf (2, "  Message 1 completed\n");
-	if (msg->status_code != SOUP_STATUS_CANCELLED) {
-		debug_printf (1, "  Unexpected status on Message 1: %d %s\n",
-			      msg->status_code, msg->reason_phrase);
-		errors++;
-	}
+	soup_test_assert_message_status (msg, SOUP_STATUS_CANCELLED);
 	g_main_loop_quit (loop);
 }
 
@@ -485,7 +417,8 @@ do_early_abort_test (void)
 	GMainContext *context;
 	GMainLoop *loop;
 
-	debug_printf (1, "\nAbort with pending connection (msg api)\n");
+	g_test_bug ("596074");
+	g_test_bug ("618641");
 
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
 	msg = soup_message_new_from_uri ("GET", base_uri);
@@ -509,11 +442,7 @@ do_early_abort_test (void)
 	soup_session_send_message (session, msg);
 	debug_printf (2, "  Message 2 completed\n");
 
-	if (msg->status_code != SOUP_STATUS_CANCELLED) {
-		debug_printf (1, "    Unexpected response: %d %s\n",
-			      msg->status_code, msg->reason_phrase);
-		errors++;
-	}
+	soup_test_assert_message_status (msg, SOUP_STATUS_CANCELLED);
 	g_object_unref (msg);
 
 	while (g_main_context_pending (context))
@@ -521,6 +450,8 @@ do_early_abort_test (void)
 
 	soup_test_session_abort_unref (session);
 
+	g_test_bug ("668098");
+
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
 	msg = soup_message_new_from_uri ("GET", base_uri);
 
@@ -529,11 +460,7 @@ do_early_abort_test (void)
 	soup_session_send_message (session, msg);
 	debug_printf (2, "  Message 3 completed\n");
 
-	if (msg->status_code != SOUP_STATUS_CANCELLED) {
-		debug_printf (1, "    Unexpected response: %d %s\n",
-			      msg->status_code, msg->reason_phrase);
-		errors++;
-	}
+	soup_test_assert_message_status (msg, SOUP_STATUS_CANCELLED);
 	g_object_unref (msg);
 
 	while (g_main_context_pending (context))
@@ -548,14 +475,8 @@ ear_one_completed (GObject *source, GAsyncResult *result, gpointer user_data)
 	GError *error = NULL;
 
 	debug_printf (2, "  Request 1 completed\n");
-	if (soup_request_send_finish (SOUP_REQUEST (source), result, &error)) {
-		debug_printf (1, "  Request 1 succeeded?\n");
-		errors++;
-	} else if (!g_error_matches (error, SOUP_HTTP_ERROR, SOUP_STATUS_CANCELLED)) {
-		debug_printf (1, "  Unexpected error on Request 1: %s\n",
-			      error->message);
-		errors++;
-	}
+	soup_request_send_finish (SOUP_REQUEST (source), result, &error);
+	g_assert_error (error, SOUP_HTTP_ERROR, SOUP_STATUS_CANCELLED);
 	g_clear_error (&error);
 }
 
@@ -565,14 +486,8 @@ ear_two_completed (GObject *source, GAsyncResult *result, gpointer loop)
 	GError *error = NULL;
 
 	debug_printf (2, "  Request 2 completed\n");
-	if (soup_request_send_finish (SOUP_REQUEST (source), result, &error)) {
-		debug_printf (1, "  Request 2 succeeded?\n");
-		errors++;
-	} else if (!g_error_matches (error, SOUP_HTTP_ERROR, SOUP_STATUS_CANCELLED)) {
-		debug_printf (1, "  Unexpected error on Request 2: %s\n",
-			      error->message);
-		errors++;
-	}
+	soup_request_send_finish (SOUP_REQUEST (source), result, &error);
+	g_assert_error (error, SOUP_HTTP_ERROR, SOUP_STATUS_CANCELLED);
 	g_clear_error (&error);
 
 	g_main_loop_quit (loop);
@@ -584,14 +499,8 @@ ear_three_completed (GObject *source, GAsyncResult *result, gpointer loop)
 	GError *error = NULL;
 
 	debug_printf (2, "  Request 3 completed\n");
-	if (soup_request_send_finish (SOUP_REQUEST (source), result, &error)) {
-		debug_printf (1, "  Request 3 succeeded?\n");
-		errors++;
-	} else if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-		debug_printf (1, "  Unexpected error on Request 3: %s\n",
-			      error->message);
-		errors++;
-	}
+	soup_request_send_finish (SOUP_REQUEST (source), result, &error);
+	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CANCELLED);
 	g_clear_error (&error);
 
 	g_main_loop_quit (loop);
@@ -613,8 +522,6 @@ do_early_abort_req_test (void)
 	GMainLoop *loop;
 	GCancellable *cancellable;
 
-	debug_printf (1, "\nAbort with pending connection (request api)\n");
-
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC,
 					 SOUP_SESSION_USE_THREAD_CONTEXT, TRUE,
 					 NULL);
@@ -683,21 +590,10 @@ do_one_accept_language_test (const char *language, const char *expected_header)
 	soup_session_send_message (session, msg);
 	soup_test_session_abort_unref (session);
 
-	if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
-		debug_printf (1, "    Message failed? %d %s\n",
-			      msg->status_code, msg->reason_phrase);
-		errors++;
-	}
+	soup_test_assert_message_status (msg, SOUP_STATUS_OK);
 	val = soup_message_headers_get_list (msg->request_headers,
 					     "Accept-Language");
-	if (!val) {
-		debug_printf (1, "    No Accept-Language set!\n");
-		errors++;
-	} else if (strcmp (val, expected_header) != 0) {
-		debug_printf (1, "    Wrong Accept-Language: expected '%s', got '%s'\n",
-			      expected_header, val);
-		errors++;
-	}
+	g_assert_cmpstr (val, ==, expected_header);
 
 	g_object_unref (msg);
 }
@@ -707,7 +603,7 @@ do_accept_language_test (void)
 {
 	const char *orig_language;
 
-	debug_printf (1, "\nAutomatic Accept-Language processing\n");
+	g_test_bug ("602547");
 
 	orig_language = g_getenv ("LANGUAGE");
 	do_one_accept_language_test ("C", "en");
@@ -780,11 +676,7 @@ do_cancel_while_reading_test_for_session (SoupSession *session)
 	while (!done)
 		g_main_context_iteration (NULL, TRUE);
 
-	if (msg->status_code != SOUP_STATUS_CANCELLED) {
-		debug_printf (1, "      FAILED: %d %s (expected Cancelled)\n",
-			      msg->status_code, msg->reason_phrase);
-		errors++;
-	}
+	soup_test_assert_message_status (msg, SOUP_STATUS_CANCELLED);
 	g_object_unref (msg);
 
 	if (thread)
@@ -796,7 +688,8 @@ do_cancel_while_reading_test (void)
 {
 	SoupSession *session;
 
-	debug_printf (1, "\nCancelling message while reading response (msg api)\n");
+	g_test_bug ("637741");
+	g_test_bug ("676038");
 
 	debug_printf (1, "  Async session\n");
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
@@ -824,14 +717,7 @@ do_cancel_while_reading_req_test_for_session (SoupSession *session,
 
 	cancellable = g_cancellable_new ();
 	soup_test_request_send (req, cancellable, flags, &error);
-	if (!error) {
-		debug_printf (1, "  Request succeeded?\n");
-		errors++;
-	} else if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-		debug_printf (1, "  Unexpected error: %s\n",
-			      error->message);
-		errors++;
-	}
+	g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CANCELLED);
 	g_clear_error (&error);
 
 	g_object_unref (req);
@@ -839,12 +725,13 @@ do_cancel_while_reading_req_test_for_session (SoupSession *session,
 }
 
 static void
-do_cancel_while_reading_req_test (void)
+do_cancel_while_reading_immediate_req_test (void)
 {
 	SoupSession *session;
 	guint flags;
 
-	debug_printf (1, "\nCancelling (immediately) message while reading response (request api)\n");
+	g_test_bug ("692310");
+
 	flags = SOUP_TEST_REQUEST_CANCEL_CANCELLABLE | SOUP_TEST_REQUEST_CANCEL_IMMEDIATE;
 
 	debug_printf (1, "  Async session\n");
@@ -859,8 +746,14 @@ do_cancel_while_reading_req_test (void)
 					 NULL);
 	do_cancel_while_reading_req_test_for_session (session, flags);
 	soup_test_session_abort_unref (session);
+}
+
+static void
+do_cancel_while_reading_delayed_req_test (void)
+{
+	SoupSession *session;
+	guint flags;
 
-	debug_printf (1, "\nCancelling (after 100ms) message while reading response (request api)\n");
 	flags = SOUP_TEST_REQUEST_CANCEL_CANCELLABLE | SOUP_TEST_REQUEST_CANCEL_SOON;
 
 	debug_printf (1, "  Async session\n");
@@ -875,8 +768,16 @@ do_cancel_while_reading_req_test (void)
 					 NULL);
 	do_cancel_while_reading_req_test_for_session (session, flags);
 	soup_test_session_abort_unref (session);
+}
+
+static void
+do_cancel_while_reading_preemptive_req_test (void)
+{
+	SoupSession *session;
+	guint flags;
+
+	g_test_bug ("637039");
 
-	debug_printf (1, "\nCancelling (preemptively) message while reading response (request api)\n");
 	flags = SOUP_TEST_REQUEST_CANCEL_CANCELLABLE | SOUP_TEST_REQUEST_CANCEL_PREEMPTIVE;
 
 	debug_printf (1, "  Async session\n");
@@ -910,20 +811,11 @@ do_aliases_test_for_session (SoupSession *session,
 
 	redirected_protocol = soup_message_headers_get_one (msg->response_headers, "X-Redirected-Protocol");
 
-	if (g_strcmp0 (redirect_protocol, redirected_protocol)) {
-		debug_printf (1, "    redirect went to %s, should have gone to %s!\n",
-			      redirected_protocol ? redirected_protocol : "(none)",
-			      redirect_protocol ? redirect_protocol : "(none)");
-		errors++;
-	} else if (redirect_protocol && !SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
-		debug_printf (1, "    msg failed? (%d %s)\n",
-			      msg->status_code, msg->reason_phrase);
-		errors++;
-	} else if (!redirect_protocol && SOUP_STATUS_IS_SUCCESSFUL (msg->status_code)) {
-		debug_printf (1, "    msg succeeded? (%d %s)\n",
-			      msg->status_code, msg->reason_phrase);
-		errors++;
-	}
+	g_assert_cmpstr (redirect_protocol, ==, redirected_protocol);
+	if (redirect_protocol)
+		soup_test_assert_message_status (msg, SOUP_STATUS_OK);
+	else
+		soup_test_assert_message_status (msg, SOUP_STATUS_FOUND);
 
 	g_object_unref (msg);
 }
@@ -934,8 +826,6 @@ do_aliases_test (void)
 	SoupSession *session;
 	char *aliases[] = { "foo", NULL };
 
-	debug_printf (1, "\nhttp-aliases / https-aliases\n");
-
 	debug_printf (1, "  Default behavior\n");
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
 	do_aliases_test_for_session (session, "http");
@@ -966,7 +856,7 @@ do_idle_on_dispose_test (void)
 	SoupMessage *msg;
 	GMainContext *async_context;
 
-	debug_printf (1, "\nTesting SoupSessionAsync dispose behavior\n");
+	g_test_bug ("667364");
 
 	async_context = g_main_context_new ();
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC,
@@ -982,10 +872,8 @@ do_idle_on_dispose_test (void)
 
 	g_object_run_dispose (G_OBJECT (session));
 
-	if (g_main_context_iteration (async_context, FALSE)) {
-		debug_printf (1, "  idle was queued!\n");
-		errors++;
-	}
+	if (g_main_context_iteration (async_context, FALSE))
+		soup_test_assert (FALSE, "idle was queued");
 
 	g_object_unref (session);
 	g_main_context_unref (async_context);
@@ -998,7 +886,7 @@ do_pause_abort_test (void)
 	SoupMessage *msg;
 	gpointer ptr;
 
-	debug_printf (1, "\nTesting paused messages don't get leaked on abort\n");
+	g_test_bug ("673905");
 
 	session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC, NULL);
 
@@ -1009,23 +897,20 @@ do_pause_abort_test (void)
 	g_object_add_weak_pointer (G_OBJECT (msg), &ptr);
 	soup_test_session_abort_unref (session);
 
-	if (ptr) {
-		debug_printf (1, "  msg was leaked\n");
-		errors++;
-	}
+	g_assert_null (ptr);
 }
 
 int
 main (int argc, char **argv)
 {
 	SoupAuthDomain *auth_domain;
+	int ret;
 
 	test_init (argc, argv, NULL);
 
-	server = soup_test_server_new (TRUE);
+	server = soup_test_server_new (SOUP_TEST_SERVER_IN_THREAD);
 	soup_server_add_handler (server, NULL, server_callback, "http", NULL);
-	base_uri = soup_uri_new ("http://127.0.0.1/");
-	soup_uri_set_port (base_uri, soup_server_get_port (server));
+	base_uri = soup_test_server_get_uri (server, "http", NULL);
 
 	auth_domain = soup_auth_domain_basic_new (
 		SOUP_AUTH_DOMAIN_REALM, "misc-test",
@@ -1036,24 +921,27 @@ main (int argc, char **argv)
 	g_object_unref (auth_domain);
 
 	if (tls_available) {
-		ssl_server = soup_test_server_new_ssl (TRUE);
+		ssl_server = soup_test_server_new (SOUP_TEST_SERVER_IN_THREAD);
 		soup_server_add_handler (ssl_server, NULL, server_callback, "https", NULL);
-		ssl_base_uri = soup_uri_new ("https://127.0.0.1/");
-		soup_uri_set_port (ssl_base_uri, soup_server_get_port (ssl_server));
+		ssl_base_uri = soup_test_server_get_uri (ssl_server, "https", "127.0.0.1");
 	}
 
-	do_host_test ();
-	do_callback_unref_test ();
-	do_callback_unref_req_test ();
-	do_msg_reuse_test ();
-	do_early_abort_test ();
-	do_early_abort_req_test ();
-	do_accept_language_test ();
-	do_cancel_while_reading_test ();
-	do_cancel_while_reading_req_test ();
-	do_aliases_test ();
-	do_idle_on_dispose_test ();
-	do_pause_abort_test ();
+	g_test_add_func ("/misc/host", do_host_test);
+	g_test_add_func ("/misc/callback-unref/msg", do_callback_unref_test);
+	g_test_add_func ("/misc/callback-unref/req", do_callback_unref_req_test);
+	g_test_add_func ("/misc/msg-reuse", do_msg_reuse_test);
+	g_test_add_func ("/misc/early-abort/msg", do_early_abort_test);
+	g_test_add_func ("/misc/early-abort/req", do_early_abort_req_test);
+	g_test_add_func ("/misc/accept-language", do_accept_language_test);
+	g_test_add_func ("/misc/cancel-while-reading/msg", do_cancel_while_reading_test);
+	g_test_add_func ("/misc/cancel-while-reading/req/immediate", do_cancel_while_reading_immediate_req_test);
+	g_test_add_func ("/misc/cancel-while-reading/req/delayed", do_cancel_while_reading_delayed_req_test);
+	g_test_add_func ("/misc/cancel-while-reading/req/preemptive", do_cancel_while_reading_preemptive_req_test);
+	g_test_add_func ("/misc/aliases", do_aliases_test);
+	g_test_add_func ("/misc/idle-on-dispose", do_idle_on_dispose_test);
+	g_test_add_func ("/misc/pause-abort", do_pause_abort_test);
+
+	ret = g_test_run ();
 
 	soup_uri_free (base_uri);
 	soup_test_server_quit_unref (server);
@@ -1063,6 +951,5 @@ main (int argc, char **argv)
 		soup_test_server_quit_unref (ssl_server);
 	}
 
-	test_cleanup ();
-	return errors != 0;
+	return ret;
 }
